@@ -3,7 +3,7 @@ use 5.008001;
 use strict;
 use warnings;
 
-our $VERSION = '1.017';
+our $VERSION = '1.029';
 
 =head1 NAME
 
@@ -46,6 +46,7 @@ use constant OPTIONS => {
     'e|eval:s'       => 'Packing one-liner',
     'E|evalfeature:s'=> 'Packing one-liner with new syntactic features',
     'x|execute'      => 'Execute code to get dependencies',
+    'xargs:s'        => 'Args to pass when executing code',
     'X|exclude:s@'   => 'Exclude modules',
     'f|filter:s@'    => 'Input filters for scripts',
     'g|gui'          => 'No console window',
@@ -64,6 +65,7 @@ use constant OPTIONS => {
     'S|save'         => 'Preserve intermediate PAR files',
     's|sign'         => 'Sign PAR files',
     'T|tempcache:s'  => 'Temp cache name',
+    'u|unicode'      => 'Include Unicode stuff',
     'v|verbose:i'    => 'Verbosity level',
     'vv|verbose2',   => 'Verbosity level 2',
     'vvv|verbose3',  => 'Verbosity level 3',
@@ -355,7 +357,7 @@ sub _create_valid_hash {
     return () if (%$hashout);
 
     foreach my $key (keys(%$hashin)) {
-        my (@keys) = ($key =~ /\|/) ? ($key =~ /(?<!:)(\w+)/g) : ($key);
+        my (@keys) = $key =~ /(?<!:)(\w+)/g;
         @{$hashout}{@keys} = ($hashin->{$key}) x @keys;
     }
 }
@@ -542,6 +544,7 @@ sub _make_manifest {
     my ($self) = @_;
 
     my $full_manifest = $self->{full_manifest};
+    my $add_manifest  = $self->{add_manifest};
 
     my $opt      = $self->{options};
     my $par_file = $self->{par_file};
@@ -553,7 +556,7 @@ sub _make_manifest {
 
     my $manifest = join("\n",
 '    <!-- accessible as jar:file:///NAME.par!/MANIFEST in compliant browsers -->',
-        (sort keys %$full_manifest),
+        (sort keys %$full_manifest, keys %$add_manifest),
 q(    # <html><body onload="var X=document.body.innerHTML.split(/\n/);var Y='<iframe src=&quot;META.yml&quot; style=&quot;float:right;height:40%;width:40%&quot;></iframe><ul>';for(var x in X){if(!X[x].match(/^\s*#/)&&X[x].length)Y+='<li><a href=&quot;'+X[x]+'&quot;>'+X[x]+'</a>'}document.body.innerHTML=Y">)
     );
 
@@ -564,7 +567,7 @@ conflicts: {}
 dist_name: $dist_name
 distribution_type: par
 dynamic_config: 0
-generated_by: '$class version $version
+generated_by: '$class version $version'
 license: unknown
 par:
   clean: $clean
@@ -673,9 +676,18 @@ sub pack_manifest_hash {
     my (@modules, @data, @exclude);
 
     foreach my $name (@{ $opt->{M} || [] }) {
+        # for "-M Foo::Bar::" also add everything below .../Foo/Bar
+        if ($name =~ s/^([\w:]+)::$/$1/) {
+            (my $mod = $name) =~ s/::/\//g;
+            Module::ScanDeps::add_preload_rule("$mod.pm", 'sub');
+        }
         $self->_name2moddata($name, \@modules, \@data);
     }
 
+    if ($opt->{u}) {
+        push @modules, "utf8_heavy.pl";
+    }
+
     # Skip either
     # a) all files from a .par file or
     # b) A module
@@ -694,7 +706,7 @@ sub pack_manifest_hash {
     my %map;
 
     unshift(@INC, @{ $opt->{I} || [] });
-    unshift(@SharedLibs, map $self->_find_shlib($_, $sn), @{ $opt->{l} || [] });
+    unshift(@SharedLibs, map $self->_find_shlib($_), @{ $opt->{l} || [] });
 
     my $inc_find = $self->_obj_function($fe, '_find_in_inc');
 
@@ -716,22 +728,27 @@ sub pack_manifest_hash {
           or $self->_die("Cannot find module $module (specified with -M)\n");
         push @files, $file;
         
-        my $preload = Module::ScanDeps::_get_preload($module) or next;
+        my @preload = Module::ScanDeps::_get_preload($module) or next;
         
         $add_deps->(
             used_by => $file,
             rv      => \%map,
-            modules => $preload,
+            modules => \@preload,
             skip    => \%skip,
 #            warn_missing => $args->{warn_missing},
         );
-        push @files, map {&$inc_find($_)} @$preload;
+        push @files, map {&$inc_find($_)} @preload;
     }
     push @files, @$input;
 
     # Search for scannable code in all -I'd paths
     push @Module::ScanDeps::IncludeLibs, @{$opt->{I}} if $opt->{I};
     
+    if ($opt->{x} && defined $opt->{xargs}) {
+        require Text::ParseWords;
+        $opt->{x} = [ Text::ParseWords::shellwords($opt->{xargs}) ];
+    }
+
     my $scan_dispatch =
       $opt->{n}
       ? $self->_obj_function($fe, 'scan_deps_runtime')
@@ -754,7 +771,7 @@ sub pack_manifest_hash {
     %skip = map { $_, 1 } map &$inc_find($_), @exclude;
     %skip = (%skip, map { $_, 1 } @SharedLibs);
 
-    &$add_deps(
+    $add_deps->(
         rv      => \%map,
         modules => \@modules,
         skip    => \%skip,
@@ -772,7 +789,7 @@ sub pack_manifest_hash {
     my $size = 0;
     my $old_member;
 
-    if ($opt->{'m'} and -e $par_file) {
+    if ($opt->{m} and -e $par_file) {
         my $tmpzip = Archive::Zip->new();
         $tmpzip->read($par_file);
 
@@ -835,6 +852,9 @@ sub pack_manifest_hash {
             $full_manifest->{"script/$name"} = [ file => $in ];
             $dep_manifest->{"script/$name"}  = [ file => $in ];
         }
+
+        delete $full_manifest->{$root . "lib/$name"};
+        delete $dep_manifest->{$root . "lib/$name"};
     }
 
     my $shlib = "shlib/$Config{archname}";
@@ -1289,25 +1309,6 @@ sub _move_parl {
     $self->{fh} = $fh;
 }
 
-sub _append_parl {
-    my ($self) = @_;
-
-    my $fh = delete $self->{fh};
-
-    my $buf;
-    seek $fh, -8, 2;
-    read $fh, $buf, 8;
-    die unless $buf eq "\nPAR.pm\n";
-    seek $fh, -12, 2;
-    read $fh, $buf, 4;
-    seek $fh, -12 - unpack("N", $buf), 2;
-
-    local $/;
-    my $cfh = $self->_open('>>', $self->{parl});
-    print $cfh readline($fh);
-    close $cfh;
-}
-
 sub _generate_output {
     my ($self) = @_;
 
@@ -1442,6 +1443,8 @@ sub _check_par {
 sub _chase_lib {
    my ($self, $file) = @_;
 
+   return $self->_chase_lib_darwin($file) if $^O eq q/darwin/;
+
    while ($Config::Config{d_symlink} and -l $file) {
        if ($file =~ /^(.*?\.\Q$Config{dlext}\E\.\d+)\..*/) {
            return $1 if -e $1;
@@ -1464,8 +1467,36 @@ sub _chase_lib {
    return $file;
 }
 
+sub _chase_lib_darwin {
+   my ($self, $file) = @_;
+
+   while (-l $file) {
+       if ($file =~ /^(.*?\.\d+)(\.\d+)*\.dylib$/) {
+           my $name = $1 . q/.dylib/;
+           return $name if -e $name;
+       }
+
+       return $file if $file =~ /\D\.\d+\.dylib$/;
+
+       my $dir = File::Basename::dirname($file);
+       $file = readlink($file);
+
+       unless (File::Spec->file_name_is_absolute($file)) {
+           $file = File::Spec->rel2abs($file, $dir);
+       }
+   }
+
+   if ($file =~ /^(.*?\.\d+)(\.\d+)*\.dylib$/) {
+       my $name = $1 . q/.dylib/;
+       return $name if -e $name;
+   }
+
+   return $file;
+}
+
+
 sub _find_shlib {
-    my ($self, $file, $script_name) = @_;
+    my ($self, $file) = @_;
 
     if ($^O eq 'MSWin32') {
         if ($file !~ /^[a-z]:/i) {
@@ -1486,25 +1517,41 @@ sub _find_shlib {
                            ? $ENV{ $Config{ldlibpthname} } : undef;
     }
     if (not defined $libpthname) {
-        print "Can't find $file. Environment variable "
-          . ($^O eq 'MSWin32' ? 'PATH' : $Config{ldlibpthname})
-          . " does not exist.\n";
+        $self->_die(sprintf(
+                "Don't know how to find shared library (option -l) %s: ".
+                "environment variable %s is not set.",
+                $file, $^O eq 'MSWin32' ? 'PATH' : $Config{ldlibpthname}));
         return;
     }
 
     $file = File::Basename::basename($file);
-    for my $dir (File::Basename::dirname($0),
-        split(/\Q$Config{path_sep}\E/, $libpthname))
-    {
-        my $abs = File::Spec->catfile($dir, $file);
-        return $self->_chase_lib($abs) if -e $abs;
-        $abs = File::Spec->catfile($dir, "$file.$Config{dlext}");
-        return $self->_chase_lib($abs) if -e $abs;
-    }
+    my @path = (File::Basename::dirname($0), split(/\Q$Config{path_sep}\E/, $libpthname));
+
+    my $lib = $self->_find_shlib_in_path($file, @path);
+    return $lib if $lib;
+
+    $self->_die("Can't find shared library (option -l): $file")
+        if $^O eq 'MSWin32' || $file =~ /^lib/;
 
     # be extra magical and prepend "lib" to the filename
-    return if $^O eq 'MSWin32';
-    return $self->_find_shlib("lib$file", $script_name) unless $file =~ /^lib/;
+    $lib = $self->_find_shlib_in_path("lib$file", @path);
+    return $lib if $lib;
+
+    $self->_die("Can't find shared library (option -l): $file (also tried lib$file)");
+}
+
+sub _find_shlib_in_path
+{
+    my ($self, $file, @path) = @_;
+
+    my $dlext = $^O eq 'darwin' ? 'dylib' : $Config{dlext};
+    for my $dir (@path)
+    {
+        my $abs = File::Spec->catfile($dir, $file);
+        $abs = File::Spec->catfile($dir, "$file.$dlext") unless -e $abs;
+        return $self->_chase_lib($abs) if -e $abs;
+    }
+    return;
 }
 
 sub _can_run {
@@ -1673,7 +1720,7 @@ Audrey Tang E<lt>cpan@audreyt.orgE<gt>
 
 Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
-L<http://par.perl.org/> is the official PAR website.  You can write
+You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
 
