@@ -241,14 +241,29 @@ my ($start_pos, $data_pos);
     open _FH, '<', $progname or last;
     binmode(_FH);
 
+    # Search for the "\nPAR.pm\n signature backward from the end of the file
     my $buf;
-    seek _FH, -8, 2;
-    read _FH, $buf, 8;
-    last unless $buf eq "\nPAR.pm\n";
+    my $size = -s $progname;
+    my $offset = 512;
+    my $idx = -1;
+    while (1)
+    {
+        $offset = $size if $offset > $size;
+        seek _FH, -$offset, 2 or die qq[seek failed on "$progname": $!];
+        my $nread = read _FH, $buf, $offset;
+        die qq[read failed on "$progname": $!] unless $nread == $offset;
+        $idx = rindex($buf, "\nPAR.pm\n");
+        last if $idx >= 0 || $offset == $size || $offset > 128 * 1024;
+        $offset *= 2;
+    }
+    last unless $idx >= 0;
 
-    seek _FH, -12, 2;
+    # Seek 4 bytes backward from the signature to get the offset of the 
+    # first embedded FILE, then seek to it
+    $offset -= $idx - 4;
+    seek _FH, -$offset, 2;
     read _FH, $buf, 4;
-    seek _FH, -12 - unpack("N", $buf), 2;
+    seek _FH, -$offset - unpack("N", $buf), 2;
     read _FH, $buf, 4;
 
     $data_pos = (tell _FH) - 4;
@@ -464,6 +479,10 @@ if ($out) {
 
 
     if (defined $par) {
+        # increase the chunk size for Archive::Zip so that it will find the EOCD
+        # even if more stuff has been appended to the .par
+        Archive::Zip::setChunkSize(128*1024);
+
         open my $fh, '<', $par or die "Cannot find '$par': $!";
         binmode($fh);
         bless($fh, 'IO::File');
@@ -756,6 +775,7 @@ sub require_modules {
     eval { require Archive::Unzip::Burst };
     eval { require Tie::Hash::NamedCapture };
     eval { require PerlIO; require PerlIO::scalar };
+    eval { require utf8 };
 }
 
 # The C version of this code appears in myldr/mktmpdir.c
@@ -978,7 +998,7 @@ L<PAR>, L<PAR::Dist>, L<parl>, L<pp>
 Audrey Tang E<lt>cpan@audreyt.orgE<gt>,
 Steffen Mueller E<lt>smueller@cpan.orgE<gt>
 
-L<http://par.perl.org/> is the official PAR website.  You can write
+You can write
 to the mailing list at E<lt>par@perl.orgE<gt>, or send an empty mail to
 E<lt>par-subscribe@perl.orgE<gt> to participate in the discussion.
 
